//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Job/ParameterTreeUtils.h
//! @brief     Defines namespace GUI::Model::ParameterTreeUtils
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_JOB_PARAMETERTREEUTILS_H
#define BORNAGAIN_GUI_MODEL_JOB_PARAMETERTREEUTILS_H

#include <QPair>
#include <QString>
#include <functional>

class JobItem;
class ParameterItem;
class ParameterLabelItem;
class ParameterContainerItem;
class DoubleDescriptor;
class VectorDescriptor;
class ParticleLayoutItem;
class ItemWithParticles;

//! The ParameterTreeBuilder contains helper functions to create container
//! with ParameterItems. The ParameterItem appears in RealTimeView and provides real
//! time tuning of MultiLayerItem and InstrumentItem.

class ParameterTreeBuilder {
public:
    ParameterTreeBuilder(JobItem* jobItem, bool recreateBackupValues);

    void build();

private:
    //! add the job's materials
    void addMaterials();

    //! add the job's sample
    void addSample();
    void addInstrument();
    void addParameterItem(ParameterLabelItem* parent, DoubleDescriptor d);
    void addParameterItem(ParameterLabelItem* parent, VectorDescriptor d);
    ParameterContainerItem* parameterContainer();
    bool allowMagneticFields() const;

    void addInterference(ParameterLabelItem* layoutLabel, const ParticleLayoutItem* layout);
    void addParticle(ParameterLabelItem* parentLabel, ItemWithParticles* p);

private:
    JobItem* m_jobItem;
    bool m_recreateBackupValues;
};

#endif // BORNAGAIN_GUI_MODEL_JOB_PARAMETERTREEUTILS_H
